﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

using NotBlackMagic;

namespace MotorTestStand {
	public partial class MainWindow : Window {
		STMDAQ daq = new STMDAQ();

		bool enableValueChangedEvents = false;

		double thrustTARA = 0;

		const int dataLength = 512;
		int dataIndex = 0;
		double[] voltageData = new double[dataLength];
		double[] currentData = new double[dataLength];
		double[] powerData = new double[dataLength];
		double[] thrustData = new double[dataLength];

		public MainWindow() {
			enableValueChangedEvents = false;
			InitializeComponent();
			enableValueChangedEvents = true;

			//Init GUI Update dispatcher
			DispatcherTimer dispatcherTimer = new DispatcherTimer();
			dispatcherTimer.Interval = TimeSpan.FromMilliseconds(500);
			dispatcherTimer.Tick += GUIUpdateHandler;
			dispatcherTimer.Start();

			//Configure Chart
			CartesianChart.plt.PlotScatter(powerData, thrustData, color: System.Drawing.Color.Red);

			//Set Axis Scale
			CartesianChart.plt.Axis(0, 20, 0, 75);

			//Other Configs
			CartesianChart.Configure(
				enablePanning: false,
				enableRightClickZoom: false,
				enableScrollWheelZoom: false
			);

			CartesianChart.Render();
		}

		private void OnWindowClosing(object sender, EventArgs e) {
			daq.Disconnect();
		}

		private void SerialButtonClick(object sender, RoutedEventArgs e) {
			if (daq.IsConnected() == false) {
				if (daq.Connect(portNamesComboBox.Text)) {
					serialButton.Content = "Disconnect";
					serialButton.Background = Brushes.LimeGreen;

					//Init DAQ Analog Input Configurations
					daq.SetAnalogInSampligRate(1, AnalogInSampleRate.SampleRate_0k12);

					daq.AddAnalogIn(1, AnalogInRange.InputRange_15m625, AnalogInMode.Mode_Diff);		//Strain Gauge Input
					daq.AddAnalogIn(5, AnalogInRange.InputRange_125m, AnalogInMode.Mode_Diff);			//Current Sense Input
					daq.AddAnalogIn(7, AnalogInRange.InputRange_8, AnalogInMode.Mode_Diff);			//Voltage Input
				}
			}
			else {
				daq.Disconnect();

				serialButton.Content = "Connect";
				serialButton.Background = Brushes.Red;
			}
		}

		private void NumberValidationTextBox(object sender, TextCompositionEventArgs e) {
			Regex regex = new Regex("[^0-9-]+");
			e.Handled = regex.IsMatch(e.Text);
		}

		private void GUIUpdateHandler(object sender, EventArgs e) {
			if(daq.IsConnected() == false) {
				return;
			}

			double thrust = daq.ReadAnalogInVolt(1, 100).Average();
			double current = daq.ReadAnalogInVolt(5, 100).Average();
			double voltage = daq.ReadAnalogInVolt(7, 100).Average();

			thrustRawTextBox.Text = (thrust * 1000000).ToString("000.0");

			//Scale DAQ read voltage
			double currentScaling = 0;
			double.TryParse(currentVoltScaling.Text, out currentScaling);
			current = (current * 1000);									//Scale to mV
			current = (current) / currentScaling;

			double thrustScaling = 0;
			double.TryParse(weightVoltScaling.Text, out thrustScaling);
			thrust = (thrust * 1000000);								//Scale to uV
			thrust = (thrust - thrustTARA) / thrustScaling;
			thrust = Math.Abs(thrust);

			//Calculate Power
			double power = voltage * current;

			//Update Graph Points
			voltageData[dataIndex] = voltage;
			currentData[dataIndex] = current;
			powerData[dataIndex] = power;
			thrustData[dataIndex] = thrust;

			dataIndex = dataIndex + 1;
			if(dataIndex >= dataLength) {
				dataIndex = 0;
			}

			CartesianChart.Render(skipIfCurrentlyRendering: true);

			//Update Progress Bar
			dataFillProgressBar.Value = ((double)dataIndex / dataLength) * 100;

			//Update Labels
			voltageTextBox.Text = voltage.ToString("00.00");
			currentTextBox.Text = current.ToString("00.00");
			powerTextBox.Text = power.ToString("00.00");
			thrustTextBox.Text = thrust.ToString("00.00");
		}

		private void CalThrustClick(object sender, RoutedEventArgs e) {
			thrustTARA = daq.ReadAnalogInVolt(1, 100).Average() * 1000000;
		}

		private void ClearDataClick(object sender, RoutedEventArgs e) {
			//Clear Data Arrays and reset index var
			Array.Clear(voltageData, 0, dataLength);
			Array.Clear(currentData, 0, dataLength);
			Array.Clear(powerData, 0, dataLength);
			Array.Clear(thrustData, 0, dataLength);

			dataIndex = 0;
		}

		private void SaveCSVClick(object sender, RoutedEventArgs e) {
			//Create csv .txt file (if existing overwrite)
			System.IO.StreamWriter streamWriter = new System.IO.StreamWriter(@".\data.txt");

			for (int i = 0; i < dataLength; i++) {
				streamWriter.Write(voltageData[i].ToString(CultureInfo.GetCultureInfo("en-US")) + ",");
				streamWriter.Write(currentData[i].ToString(CultureInfo.GetCultureInfo("en-US")) + ",");
				streamWriter.Write(powerData[i].ToString(CultureInfo.GetCultureInfo("en-US")) + ",");
				streamWriter.Write(thrustData[i].ToString(CultureInfo.GetCultureInfo("en-US")));
				streamWriter.Write(Environment.NewLine);
			}

			//Close file
			streamWriter.Close();
		}
	}
}
